/* common.h  -  Common definitions
 *      Copyright (C) 2002 fkt <fkt@funfatal.org>
 *
 * This file is part of ProSum.
 *
 * ProSum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ProSum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef _COMMON_H
#define _COMMON_H

#include "../config.h"

/* Comprobaciones de que se tiene lo necesario */
#if !defined(STDC_HEADERS) && !defined(IS_SUN)
  #error "ANSI C headers are required"
#endif

#ifndef TIME_WITH_SYS_TIME
  #error "Cannot include <sys/time.h> and <time.h> safely"
#endif

#ifndef HAVE_SELECT
  #error "select() is required"
#endif

#if !defined(HAVE_SOCKET) && !defined(HAVE_LIBSOCKET)
  #error "socket() is required"
#endif

#ifndef HAVE_STRFTIME
  #error "strftime() is required"
#endif

#ifndef HAVE_STRSTR
  #error "strstr() is required"
#endif

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <pwd.h>
#include <grp.h>
#include <ctype.h>
#include <syslog.h>
#include <signal.h>
#include <locale.h>
#include <libintl.h>
#include <dirent.h>
#include <termios.h>
#include <readline/readline.h>
#include <utime.h>
#include <sys/time.h>
#include <time.h>

#include "config_prosum.h"
#include "errors.h"

//Includes for encryption
#include "../gpg/include/cipher.h"

#ifdef HAVE_GETOPT_H
  #include <getopt.h>
#else
  #include "getopt.h"
#endif

#ifdef IS_SUN
  #define INADDR_NONE INADDR_BROADCAST
#endif

#ifndef HAVE_ATEXIT  /* For SunOS */
  #define atexit(a)    (on_exit((a),0))
#endif

#ifndef HAVE_STRTOUL
  #define strtoul(a,b,c)  ((unsigned long)strtol((a),(b),(c)))
#endif

#define ERROR 1
#define SUCCESS 0

#define SEND_FILE 0
#define RECV_FILE 1
#define SETUP_ALIAS 2

#define BD_ENCRYPT 1
#define NO_BD_ENCRYPT 0
#define BD_DECRYPT 1
#define NO_BD_DECRYPT 0

#ifdef HAVE_NLS
  #define _(Y) gettext(Y)
#else
  #define _(Y) (Y)
#endif

#define FATAL(X) do { \
                   perror(X); \
                   exit(ERROR); \
                 } while(0);

typedef struct {
  char filename[MAX_NAME_SZ];
  struct stat f_status; //Solo Uso 5: 1- mode 2- uid 3- gid 4- size
                        //            5- mtime
  unsigned char f_md5[16];
#ifdef HAVE_EXT2FS_EXT2_FS_H
  unsigned long ext2_flags;
#endif
} file_t;

#ifdef IDT_SUPPORT
  typedef struct {
    /* Guardo una copia de la IDT para luego simplificar si hubiera que 
       reemplazarla por la original */
    struct {
      unsigned short off1;
      unsigned short sel;
      unsigned char none, flags;
      unsigned short off2;
    } __attribute__ ((packed)) idt_intern;

    int nr_int;
    unsigned long addr;
    unsigned short segment;
    unsigned char flags;
  } idt_t;
#endif

#ifdef SYSCALL_SUPPORT
  typedef struct {
    /* 7 primeros bytes de la syscall */
    char sc_rutine[7];
    int nr_syscall;
    unsigned long addr;
  } syscall_t;
#endif

#ifdef KSYMS_SUPPORT
  typedef struct {
    /* 7 primeros bytes de la rutina */
    char ksym_rutine[7];
    unsigned long addr;
    char ksym_name[MAX_KSYM_NAME];
  } ksym_t;
#endif

typedef struct {
  file_t f_sta[MAXFILES];
  int f_validos;  
#ifdef SYSCALL_SUPPORT
  syscall_t sc_sta[NR_SYSCALLS];
  int sc_validos;
#endif
#ifdef IDT_SUPPORT
  idt_t idt_sta[NR_IDT];
  int idt_validos;
#endif
#ifdef KSYMS_SUPPORT
  ksym_t ksym_sta[NR_KSYMS];
  int ksym_validos;
#endif
} table_t;

typedef int err_buf_t[5]; // Aqui se guardara ERROR o SUCCESS en cada casilla
                          // segun lo que falle. Las 5 casillas corresponden
                          // a los que uso del struct stat

/* Codigos Servidor TCP */
static const int ok=1;
static const int error=2;
static const int overflow=3;
static const int nonext=4; //Archivo no existe
static const int alias_dup=5; //El alias ya existe en el host bastion
/* FIN Codigos Servidor TCP */

#endif /* #ifndef _COMMON_H */
