/* ext2.c  -  Extended Flags Manipulation
 *      Copyright (C) 2002 fkt <fkt@funfatal.org>
 *
 * This file is part of ProSum.
 *
 * ProSum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ProSum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

/* Part of code from e2fsprogs By:
 * Copyright (C) 1993, 1994  Remy Card <card@masi.ibp.fr>
 */

#include "ext2.h"

int do_ext2(file_t *f_st) {
  int fd;

  f_st->ext2_flags=0;

#if HAVE_STAT_EXT2_FLAGS
  #ifdef UF_IMMUTABLE
    if (f_st->f_status.st_flags & UF_IMMUTABLE)
      f_st->ext2_flags |= EXT2_IMMUTABLE_FL;
  #endif
  #ifdef UF_APPEND
    if (f_st->f_status.st_flags & UF_APPEND)
      f_st->ext2_flags |= EXT2_APPEND_FL;
  #endif
  #ifdef UF_NODUMP
    if (fs_st->f_status.st_flags & UF_NODUMP)
      f_st->ext2_flags |= EXT2_NODUMP_FL;
  #endif

  return SUCCESS;
#else

  if ((fd=open(f_st->filename, O_RDONLY))==-1)
    return ERROR;

  if (ioctl(fd, EXT2_IOC_GETFLAGS, &(f_st->ext2_flags))==-1) {
    close(fd);
    return ERROR;
  }

  close(fd);
  return SUCCESS;
#endif
}

int check_ext2(file_t *f_st, file_t *f_cmp) {
  if (f_st->ext2_flags != f_cmp->ext2_flags) return ERROR;

  return SUCCESS;
}

void restore_ext2(char *filename, unsigned long flags) {
#ifdef HAVE_CHFLAGS
  unsigned long flags_tmp=0;

  #ifdef UF_IMMUTABLE
    if (flags & EXT2_IMMUTABLE_FL)
      flags_tmp |= UF_IMMUTABLE;
  #endif
  #ifdef UF_APPEND
    if (flags & EXT2_APPEND_FL)
      flags_tmp |= UF_APPEND;
  #endif
  #ifdef UF_NODUMP
    if (flags & EXT2_NODUMP_FL)
      flags_tmp |= UF_NODUMP;
  #endif

  chflags(filename, flags_tmp);
#else

  int fd=open(filename, O_RDONLY);

  ioctl(fd, EXT2_IOC_SETFLAGS, &flags);

  close(fd);
#endif
}

void value_ext2(unsigned long flags, char *str) {
  if (flags & EXT2_SECRM_FL) str[0]='s';
  else str[0]='-';
  if (flags & EXT2_UNRM_FL) str[1]='u';
  else str[1]='-';
  if (flags & EXT2_SYNC_FL) str[2]='S';
  else str[2]='-';
  if (flags & EXT2_IMMUTABLE_FL) str[3]='i';
  else str[3]='-';
  if (flags & EXT2_APPEND_FL) str[4]='a';
  else str[4]='-';
  if (flags & EXT2_NODUMP_FL) str[5]='d';
  else str[5]='-';
  if (flags & EXT2_NOATIME_FL) str[6]='A';
  else str[6]='-';
  if (flags & EXT2_COMPR_FL) str[7]='c';
  else str[7]='-';
  if (flags & EXT2_COMPRBLK_FL) str[8]='B';
  else str[8]='-';
  if (flags & EXT2_DIRTY_FL) str[9]='D';
  else str[9]='-';
  if (flags & EXT2_NOCOMPR_FL) str[10]='X';
  else str[10]='-';
  if (flags & EXT2_ECOMPR_FL) str[11]='E';
  else str[11]='-';

#ifdef EXT2_JOURNAL_DATA_FL
  if (flags & EXT2_JOURNAL_DATA_FL) str[12]='j';
  else str[12]='-';
#else
  str[12]='-';
#endif

  if (flags & EXT2_NOTAIL_FL) str[13]='t';
  else str[13]='-';

  str[14]='\0';
}
