/* idt.c  -  IDT Manipulation
 *      Copyright (C) 2002 fkt <fkt@funfatal.org>
 *
 * This file is part of ProSum.
 *
 * ProSum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ProSum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "idt.h"

int put_idt(table_t *idt_tbl) {
  unsigned int count=0;
  unsigned short idt_size;
  unsigned long idt_addr_base, idt_addr;
  size_t i;
  int fd;

  if ((fd=open("/dev/kmem", O_RDONLY))==-1) return ERROR;

  idt_addr_base=get_idt_addr(&idt_size);

  for(i=0; (i < (idt_size+1)/8) && (count < NR_IDT) ; i++) {
    if (readkmem(fd,&idt,idt_addr_base+8*i,sizeof(idt))==ERROR) return ERROR;
    idt_addr=(idt.off2 << 16) | idt.off1;
    if (idt_addr != 0x0) {
      idt_tbl->idt_sta[count].nr_int=i;
      idt_tbl->idt_sta[count].flags=idt.flags;
      idt_tbl->idt_sta[count].addr=idt_addr;
      idt_tbl->idt_sta[count].segment=idt.sel;
      memcpy(&idt_tbl->idt_sta[count].idt_intern,&idt,sizeof(idt));

      count++;
    }
  }

  idt_tbl->idt_validos=count;

  close(fd);
  return SUCCESS;
}

char *value_segment(unsigned short segment) {
  switch (segment) {
    case __KERNEL_CS: return("KERNEL_CS");
                      break;
    case __KERNEL_DS: return("KERNEL_DS");
                      break;
    case __USER_CS: return("USER_CS");
                      break;
    case __USER_DS: return("USER_DS");
                      break;
    default: return("Unknown");
  }
}

int check_idt(idt_t *orig, idt_t *cmp) {
  if (orig->addr != cmp->addr) return ERROR;

  return SUCCESS;
}

int restore_idt(idt_t *st) {
  int fd;
  unsigned long idt_addr_base;

  if ((fd=open("/dev/kmem", O_WRONLY))==-1) return ERROR;

  idt_addr_base=get_idt_addr(NULL);

  if (writekmem(fd,&st->idt_intern,idt_addr_base+8*st->nr_int,sizeof(idt))==ERROR)
    return ERROR;

  close(fd);
  return SUCCESS;
}
