/* main_if.c  -  Main Utility Functions
 *      Copyright (C) 2002 fkt <fkt@funfatal.org>
 *
 * This file is part of ProSum.
 *
 * ProSum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ProSum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "main_if.h"

/***** FUNCIONES INTERNAS *****/
int setup_alias_client(int sockfd);
//Crea el fichero del alias en el bastion host, devuelve ERROR o SUCCESS

int send_cmd(int sockfd, char *cmd, unsigned int cmd_len);
//Manda el comando al host bastion. Devuelve error o ok.

int do_handshake(int sockfd);
//Hace el protocolo para saber si es un servidor prosum, si lo es devuelve ok, sino error

unsigned long resolv(char *host);
//Resuelve un host, en caso de no poder resolverlo devuelve INADDR_NONE

int check_conn(struct sockaddr_in *st_host);
//Mira a ver puede conectar al host, si puede devuelve un descriptor y si no
//devuelve ERROR

int put_file_client(int sockfd, int fd);
//Envia el fichero con descriptor fd al servidor, devuelve ERROR en caso de
//error y SUCCESS en caso satisfactorio

int get_file_client(char *cmd_wfile, int sockfd);
//Recibe el fichero del servidor, devuelve ERROR en caso de error y SUCCESS en
//caso satisfactorio

/******************************/

void init_secmem(void) {
  secmem_set_flags( secmem_get_flags() | 2 );
  secure_random_alloc();
  secmem_init(16384);
}

int check_lock_file(char *file) {
  struct stat stat_buf;

  if (stat(file,&stat_buf)==-1) return SUCCESS;

  return ERROR;
}

int create_lock_file(char *lfile) {
  FILE *f;
  pid_t p=getpid();

  if ((f=fopen(lfile,"wt"))==NULL)
    return ERROR;

  fprintf(f,"%d",p);

  fclose(f);
  return SUCCESS;
}

int is_symbolic_link(char *file) {
  struct stat stat_buf;

  if (lstat(file,&stat_buf)==-1) return ERROR;

  if (!S_ISREG(stat_buf.st_mode)) return SUCCESS;

  return ERROR;
}

int file_srv(char *file, int cmd_opt) {
  struct sockaddr_in st_bastion_host;
  int bastion_file_fd, sockfd, code;
  char comando[MAX_CMD];

  //Miramos si el archivo lo podemos abrir en caso de que haya que mandarlo
  if (cmd_opt==SEND_FILE)
    if ((bastion_file_fd=open(file, O_RDONLY))==-1)
      return ERROR;

  //Conectamos al host bastion
  if ((st_bastion_host.sin_addr.s_addr=resolv(TCPSERV_NAME))==INADDR_NONE)
    return ERROR;

  st_bastion_host.sin_family=AF_INET;
#ifdef WORDS_BIGENDIAN
  st_bastion_host.sin_port=TCPSERV_PORT;
#else
  st_bastion_host.sin_port=htons(TCPSERV_PORT);
#endif
  if ((sockfd=check_conn(&st_bastion_host))==ERROR)
    return ERROR;

  if (do_handshake(sockfd)==error) {
    errno=ENOVERSION;
    close(sockfd);
    return ERROR;
  }

  //Construyo el comando
  switch (cmd_opt) {
    case SEND_FILE:
             snprintf(comando,MAX_CMD,"put %s\r\n", file);
             break;

    case RECV_FILE:
             snprintf(comando,MAX_CMD,"get %s\r\n", file);
             break;

    case SETUP_ALIAS:
             snprintf(comando,MAX_CMD,"setup %s\r\n", file);
             break;
  }

  if (send_cmd(sockfd,comando,strlen(comando))==error) {
    close(sockfd);
    errno=ENOCMD;
    return ERROR;
  }

  if (cmd_opt != SETUP_ALIAS) {
    //Mando el alias y miro a ver si no recibo error
    write(sockfd,ALIAS_NAME,strlen(ALIAS_NAME));
    read(sockfd,&code,sizeof(int));
    if (code==error) {
      errno=ENOALIAS;
      close(sockfd);
      return ERROR;
    }
  }

  //Quito el \r\n del final
  comando[strlen(comando)-2]='\0';

  switch (cmd_opt) {
    case SEND_FILE:
      if (put_file_client(sockfd,bastion_file_fd)==ERROR) {
        errno=ENOSEND;
        close(sockfd);
        return ERROR;
      }

      break;

   case RECV_FILE:
      if (get_file_client(comando,sockfd)==ERROR) {
        errno=ENORECV;
        close(sockfd);
        return ERROR;
      }

      break;

    case SETUP_ALIAS:
      if (setup_alias_client(sockfd)==ERROR) {
        errno=ENODUPA;
        close(sockfd);
        return ERROR;
      }

      break;
  }

  close(sockfd);
  return SUCCESS;
}

int setup_alias_client(int sockfd) {
  int code;

  read(sockfd,&code,sizeof(int));

  if (code != ok) return ERROR;

  return SUCCESS;
}

int send_cmd(int sockfd, char *cmd, unsigned int cmd_len) {
  int code;

  write(sockfd,cmd,cmd_len);
  read(sockfd,&code,sizeof(int));

  if (code==error) return error;

  return ok;
}

int do_handshake(int sockfd) {
  int code;

  write(sockfd,"ProSum_0.28",12);
  read(sockfd,&code,sizeof(int));

  if (code==error) return error;
  else if (code != ok) return error;

  return ok;
}

int check_conn(struct sockaddr_in *st_host) {
  int sockfd;
  int flags, error, n;
  socklen_t elen, len=sizeof(struct sockaddr_in);
  fd_set rset, wset;
  struct timeval time;

  if ((sockfd=socket(AF_INET,SOCK_STREAM,0))==-1) return ERROR;

  if ((flags=fcntl(sockfd, F_GETFL, 0))==-1) return ERROR;

  if (fcntl(sockfd, F_SETFL, flags | O_NONBLOCK)==-1) return ERROR;

  error=0;
  if ((n=connect(sockfd, (struct sockaddr *)st_host, len)) < 0)
    if (errno != EINPROGRESS) return ERROR;

  if (!n) goto done;

  FD_ZERO(&rset);
  FD_SET(sockfd, &rset);
  wset=rset;

  time.tv_sec=TCPSERV_TIMEOUT;
  time.tv_usec=0;

  if ((n=select(sockfd + 1, &rset, &wset, NULL, TCPSERV_TIMEOUT ? &time : NULL))==0) {
    close(sockfd);
    errno=ETIMEDOUT;
    return ERROR;
  }

  if (FD_ISSET(sockfd, &rset) || FD_ISSET(sockfd, &wset)) {
    elen=sizeof(error);
    if (getsockopt(sockfd, SOL_SOCKET, SO_ERROR, &error, &elen) < 0) {
      return ERROR;
    }
  }

  done:
    if (fcntl(sockfd, F_SETFL, 0)==-1) return ERROR;

    if (error) {
      close(sockfd);
      errno=error;
      return ERROR;
    }

    return sockfd;
}

void do_questions(int *replace, int *req_change, int changed, int automatic,
                  int interactive) {
  char *line=NULL, answer;

  if (changed && interactive && (!automatic)) {
    do {
      free(line);

      printf(_("\nIf this change/s were intencionally answer Y\n"));
      printf(_("\nYou Want To Save This Change/s? (Y/N): "));

      line=readline(NULL);
      sscanf(line,"%c", &answer);
    } while ((toupper(answer) != 'Y') && (toupper(answer) != 'N'));

    if (toupper(answer)=='Y') *req_change=1;
    else { //Si responde con un N
      do {
        free(line);

        printf(_("\nYou Want To Restaure Changes? (Y/N): "));

        line=readline(NULL);
        sscanf(line,"%c", &answer);
      } while ((toupper(answer) != 'Y') && (toupper(answer) != 'N'));

      free(line);

      if (toupper(answer)=='Y') *replace=1;
    } // FIN si responde con un N
        
  } else if (changed && automatic) { //Si ha cambiado y esta automatico...
        *req_change=0;
        *replace=1;
        //MAS COSAS QUE HAYA QUE HACER SOLAS
  }
}

int put_file_client(int sockfd, int fd) {
  char buf[MAX_BUF];
  ssize_t c;
  int code;
  struct stat status;

  //Mando el modo y el tamao del fichero
  fstat(fd,&status);

  write(sockfd,&status.st_mode,sizeof(mode_t));
  write(sockfd,&status.st_size,sizeof(off_t));

  //Envio el archivo
  memset(buf,0x0,sizeof(buf));
  c=read(fd,buf,sizeof(buf));
  while (c != 0) {
    write(sockfd,buf,sizeof(buf));

    memset(buf,0x0,sizeof(buf));
    c=read(fd,buf,sizeof(buf));
  }

  //Comprobamos si se ha recibido bien el archivo
  read(sockfd,&code,sizeof(int));

  if (code==ok) return SUCCESS;
  else return ERROR;
}

int get_file_client(char *cmd_wfile, int sockfd) {
  char buf[MAX_BUF], *ultimo_recv;
  int code, fd, cont_recv, i, resto;
  mode_t mode;
  off_t file_size;
  struct stat status;

  read(sockfd,&code,sizeof(int));

  if (code==nonext) return ERROR;

  //Recibo el modo del fichero
  read(sockfd,&mode,sizeof(mode_t));
  fd=open(cmd_wfile+4, O_WRONLY | O_TRUNC | O_CREAT, mode);
  //Le cambiamos el modo, pq si el archivo ya existia no se cambia el modo solo
  fchmod(fd,mode);

  //Recibo el tamao del fichero y ajusto el contador del bucle para
  //recibir el fichero
  read(sockfd,&file_size,sizeof(off_t));

  cont_recv=file_size/MAX_BUF;

  //Recibo y escribo el fichero
  for (i=0; i < cont_recv ; i++) {
    memset(buf,0x0,sizeof(buf));
    read(sockfd,buf,sizeof(buf));
    write(fd,buf,sizeof(buf));
  }

  //Recibo y escribo lo que falte del fichero si no es multiplo de 1024
  resto=file_size%MAX_BUF;
  ultimo_recv=(char *)malloc(sizeof(char)*resto);
  read(sockfd,ultimo_recv,resto);
  write(fd,ultimo_recv,resto);

  free(ultimo_recv);
  close(fd);

  //Compruebo que el tamao del archivo que he recibido esta coincide
  //con el original
  stat(cmd_wfile+4,&status);
  if (status.st_size==file_size) {
    write(sockfd,&ok,sizeof(int));
    return SUCCESS;
  } else {
    write(sockfd,&error,sizeof(int));
    return ERROR;
  }
}

unsigned long resolv(char *name) {
  unsigned long ip;
  struct hostent *he=NULL;
  ip=INADDR_NONE;

#ifdef IS_SUN
  if (inet_pton(AF_INET, name, (struct in_addr *)&ip) <= 0) {
#else
  if (!inet_aton(name, (struct in_addr *)&ip)) {
#endif
    he=gethostbyname(name);
    if (he != NULL) memcpy (&ip, he->h_addr, sizeof (ip));
  }

  return ip;
}

//No hace falta meter en el ctrlc_hadler ni en el sigterm_handler una llamada a clean_up()
//pq ya esta resigstrada con atexit() y cuando haga un exit se ejecuta
RETSIGTYPE signal_handler(int signum) {
  printf(_("\nSignal Number %d Cached!, Cleaning Up...\n"), signum);

  exit(SUCCESS);
}

void clean_up(void) {
  char filedir[MAX_NAME_SZ];
  struct dirent *f;
  DIR *tmp=opendir("/tmp/");

  f=readdir(tmp);
  while (f != NULL) {
    if (strstr(f->d_name,"prosum")) {
      memset(filedir,0x0,MAX_NAME_SZ);
      strcat(filedir,"/tmp/");
      strncat(filedir,f->d_name,MAX_NAME_SZ-6);
      unlink(filedir);

      memset(filedir,0x0,sizeof(filedir));
    }

    f=readdir(tmp);
  }

  closedir(tmp);

  unlink(LOCK_FILE);
}

int encrypt_bd(char *filename, CIPHER_HANDLE alg) {
  int fd_encrypt, fd_plain;
  ssize_t c;
  char buf[MAX_BUF], tmp_file[MAX_NAME_SZ];

  snprintf(tmp_file,MAX_NAME_SZ,"%s.%d","/tmp/prosum_enc_bf",getpid());

  if (copy_file(filename,tmp_file)==ERROR) return ERROR;

  if ((fd_plain=open(tmp_file,O_RDONLY))==-1) return ERROR;

  if ((fd_encrypt=open(filename,O_WRONLY | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR))==-1) return ERROR;

  memset(buf,0x0,sizeof(buf));

  c=read(fd_plain,buf,MAX_BUF);
  while (c > 0) {
    cipher_encrypt(alg, buf, buf, c);

    write(fd_encrypt, buf, c);

    memset(buf,0x0,sizeof(buf));
    c=read(fd_plain,buf,MAX_BUF);
  }

  close(fd_plain);
  close(fd_encrypt);
  if (unlink(tmp_file)==-1) return ERROR;

  return SUCCESS;
}

int decrypt_bd(char *filename, CIPHER_HANDLE alg) {
  int fd_encrypt, fd_plain;
  ssize_t c;
  char tmp_file[MAX_NAME_SZ], buf[MAX_BUF];

  snprintf(tmp_file,MAX_NAME_SZ,"%s.%d","/tmp/prosum_dec_bf",getpid());

  if (copy_file(filename,tmp_file)==ERROR) return ERROR;

  if ((fd_encrypt=open(tmp_file,O_RDONLY))==-1) return ERROR;

  if ((fd_plain=open(filename,O_WRONLY | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR))==-1) return ERROR;

  memset(buf,0x0,sizeof(buf));

  c=read(fd_encrypt,buf,MAX_BUF);
  while (c > 0) {
    cipher_decrypt(alg, buf, buf, c);

    write(fd_plain, buf, c);

    memset(buf,0x0,sizeof(buf));
    c=read(fd_encrypt,buf,MAX_BUF);
  }

  close(fd_plain);
  close(fd_encrypt);
  if (unlink(tmp_file)==-1) return ERROR;

  return SUCCESS;
}

void write_bd(FILE *fp_bd, table_t *f_tb) {
  size_t i;

  fwrite(&f_tb->f_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->f_validos ; i++)
    fwrite(&f_tb->f_sta[i],sizeof(file_t),1,fp_bd);

#ifdef KSYMS_SUPPORT
  fwrite(&f_tb->ksym_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->ksym_validos ; i++)
    fwrite(&f_tb->ksym_sta[i],sizeof(ksym_t),1,fp_bd);
#endif

#ifdef IDT_SUPPORT
  fwrite(&f_tb->idt_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->idt_validos ; i++)
    fwrite(&f_tb->idt_sta[i],sizeof(idt_t),1,fp_bd);
#endif

#ifdef SYSCALL_SUPPORT
  fwrite(&f_tb->sc_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->sc_validos ; i++)
    fwrite(&f_tb->sc_sta[i],sizeof(syscall_t),1,fp_bd);
#endif
}

void read_bd(FILE *fp_bd, table_t *f_tb) {
  size_t i;

  fread(&f_tb->f_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->f_validos; i++)
    fread(&f_tb->f_sta[i],sizeof(file_t),1,fp_bd);

#ifdef KSYMS_SUPPORT
  fread(&f_tb->ksym_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->ksym_validos ; i++)
    fread(&f_tb->ksym_sta[i],sizeof(ksym_t),1,fp_bd);
#endif

#ifdef IDT_SUPPORT
  fread(&f_tb->idt_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->idt_validos ; i++)
    fread(&f_tb->idt_sta[i],sizeof(idt_t),1,fp_bd);
#endif

#ifdef SYSCALL_SUPPORT
  fread(&f_tb->sc_validos,sizeof(int),1,fp_bd);
  for(i=0; i < f_tb->sc_validos ; i++)
    fread(&f_tb->sc_sta[i],sizeof(syscall_t),1,fp_bd);
#endif
}

void supr_retorno_carro(char *buf) {
  char *ptr;

  for(ptr=buf; *ptr != '\0' ; ptr++)
    ;

  if (*(ptr-1) == '\n') *(ptr-1)='\0';
}

void prosum_usage(char *prog) {
  printf(_("\n%s %s By fkt <fkt@funfatal.org>\n"), PACKAGE, VERSION);
  printf(_("Usage: %s [options] -p <password>\n\n"), prog);
  printf(_("options:\n"));
  printf(_("-v <bbdd_file>"));
  #ifdef  KSYMS_SUPPORT
    printf(_(" -f <config_file>"));
  #endif
  printf(_(" [-d <key> -s <seed>] [-i] [-a] : Verify files that are on BBDD, you can, or not, put -i to "
           "interactive with program, to decide what do when a change is found\n"));
  printf(_("-c <files_file> <bbdd_file>"));
  #ifdef KSYMS_SUPPORT
    printf(_(" -f <config_file>"));
  #endif
  printf(_(" [-e <key> -s <seed>] : Create a new BBDD file with files that contains files_file\n"));
  printf(_("-r <bbdd_file> [-d <key> -s <seed>] : Reports to stdout the contain of BBDD\n\n"));
  printf(_("NOTE: With -c option you can put -e <key> -s <seed> to encrypt BBDD with BlowFish\n"));
  printf(_("      And if you encrypt BBDD you must put -d <key> -s <seed> with -r and -v option\n"));
  printf(_("      to decrypt BBDD, if you don't put -d the results are unpredictable\n\n"));
  printf(_("You can append to -v option -a and the program will solve all changes automaticly\n\n"));
  exit(SUCCESS);
}

int copy_file(char *src, char *dst) {
  int fd_src, fd_dst;
  ssize_t c;
  char buf[MAX_NAME_SZ];

  if ((fd_src=open(src,O_RDONLY))==-1) return ERROR;

  if ((fd_dst=open(dst,O_WRONLY | O_TRUNC | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR))==-1)
    return ERROR;

  while((c=read(fd_src,buf,MAX_NAME_SZ)) > 0)
    write(fd_dst,buf,c);

  close(fd_src);
  close(fd_dst);

  return SUCCESS;
}

void value_mode(mode_t mode, char *mode_str) {

  if (mode & S_IRUSR) mode_str[0]='r';
  else mode_str[0]='-';
  if (mode & S_IWUSR) mode_str[1]='w';
  else mode_str[1]='-';
  if ((mode & S_IXUSR) && !(mode & S_ISUID)) mode_str[2]='x';
  else if (mode & S_ISUID) mode_str[2]='s';
  else mode_str[2]='-';

  if (mode & S_IRGRP) mode_str[3]='r';
  else mode_str[3]='-';
  if (mode & S_IWGRP) mode_str[4]='w';
  else mode_str[4]='-';
  if ((mode & S_IXGRP) && !(mode & S_ISGID)) mode_str[5]='x';
  else if (mode & S_ISGID) mode_str[5]='s';
  else mode_str[5]='-';
  if (mode & S_IROTH) mode_str[6]='r';
  else mode_str[6]='-';
  if (mode & S_IWOTH) mode_str[7]='w';
  else mode_str[7]='-';
  if (mode & S_IXOTH) mode_str[8]='x';
  else mode_str[8]='-';

  mode_str[9]='\0';
}
