/* syscall.c  -  sys_call_table Manipulation
 *      Copyright (C) 2002 fkt <fkt@funfatal.org>
 *
 * This file is part of ProSum.
 *
 * ProSum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ProSum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "syscall.h"

#define CALLOFF 100
#define MAX_NAME 50

int put_syscalls(table_t *sc_tbl) {
  unsigned long sc_addr, sct, idt_addr_base, int80;
  unsigned int count=0;
  int i, fd;
  char sc_asm[CALLOFF], *p;

  idt_addr_base=get_idt_addr(NULL);

  if ((fd=open("/dev/kmem", O_RDONLY))==-1) return ERROR;

  readkmem(fd,&idt,idt_addr_base+8*0x80,sizeof(idt));
  int80=(idt.off2 << 16) | idt.off1;

  readkmem(fd,sc_asm,int80,CALLOFF);
  p=(char*)memmem(sc_asm,CALLOFF,"\xff\x14\x85",3);
  sct=*(unsigned*)(p+3);

  if (p) {
    for (i=1 ; i <= NR_SYSCALLS ; i++, count++) {
      readkmem(fd, &sc_addr, sct + i * 4, 4);
      sc_tbl->sc_sta[count].nr_syscall=i;
      sc_tbl->sc_sta[count].addr=sc_addr;
      readkmem(fd, sc_tbl->sc_sta[count].sc_rutine, sc_addr, 7);
    }
  } else {
    close(fd);
    errno=ENOSYSCALL;
    return ERROR;
  }

  sc_tbl->sc_validos=count;
  close(fd);
  return SUCCESS;
}

char *value_syscall(int id) {
  char *ret=(char *)malloc(sizeof(char)*MAX_NAME);

  switch (id) {
    case __NR_exit:  strncpy(ret,"sys_exit",MAX_NAME);
                     break;
    case __NR_fork:  strncpy(ret,"sys_fork",MAX_NAME);
                     break;
    case __NR_read:  strncpy(ret,"sys_read",MAX_NAME);
                     break;
    case __NR_write:  strncpy(ret,"sys_write",MAX_NAME);
                     break;
    case __NR_open:  strncpy(ret,"sys_open",MAX_NAME);
                     break;
    case __NR_close:  strncpy(ret,"sys_close",MAX_NAME);
                     break;
    case __NR_waitpid:  strncpy(ret,"sys_waitpid",MAX_NAME);
                     break;
    case __NR_creat:  strncpy(ret,"sys_creat",MAX_NAME);
                     break;
    case __NR_link:  strncpy(ret,"sys_link",MAX_NAME);
                     break;
    case __NR_unlink:  strncpy(ret,"sys_unlink",MAX_NAME);
                     break;
    case __NR_execve:  strncpy(ret,"sys_execve",MAX_NAME);
                     break;
    case __NR_chdir:  strncpy(ret,"sys_chdir",MAX_NAME);
                     break;
    case __NR_time:  strncpy(ret,"sys_time",MAX_NAME);
                     break;
    case __NR_mknod:  strncpy(ret,"sys_mknod",MAX_NAME);
                     break;
    case __NR_chmod:  strncpy(ret,"sys_chmod",MAX_NAME);
                     break;
    case __NR_lchown:  strncpy(ret,"sys_lchown",MAX_NAME);
                     break;
    case __NR_break:  strncpy(ret,"sys_break",MAX_NAME);
                     break;
    case __NR_oldstat:  strncpy(ret,"sys_oldstat",MAX_NAME);
                     break;
    case __NR_lseek:  strncpy(ret,"sys_lseek",MAX_NAME);
                     break;
    case __NR_getpid:  strncpy(ret,"sys_getpid",MAX_NAME);
                     break;
    case __NR_mount:  strncpy(ret,"sys_mount",MAX_NAME);
                     break;
    case __NR_umount:  strncpy(ret,"sys_umount",MAX_NAME);
                     break;
    case __NR_setuid:  strncpy(ret,"sys_setuid",MAX_NAME);
                     break;
    case __NR_getuid:  strncpy(ret,"sys_getuid",MAX_NAME);
                     break;
    case __NR_stime:  strncpy(ret,"sys_stime",MAX_NAME);
                     break;
    case __NR_ptrace:  strncpy(ret,"sys_ptrace",MAX_NAME);
                     break;
    case __NR_alarm:  strncpy(ret,"sys_alarm",MAX_NAME);
                     break;
    case __NR_oldfstat:  strncpy(ret,"sys_oldfstat",MAX_NAME);
                     break;
    case __NR_pause:  strncpy(ret,"sys_pause",MAX_NAME);
                     break;
    case __NR_utime:  strncpy(ret,"sys_utime",MAX_NAME);
                     break;
    case __NR_stty:  strncpy(ret,"sys_stty",MAX_NAME);
                     break;
    case __NR_gtty:  strncpy(ret,"sys_gtty",MAX_NAME);
                     break;
    case __NR_access:  strncpy(ret,"sys_access",MAX_NAME);
                     break;
    case __NR_nice:  strncpy(ret,"sys_nice",MAX_NAME);
                     break;
    case __NR_ftime:  strncpy(ret,"sys_ftime",MAX_NAME);
                     break;
    case __NR_sync:  strncpy(ret,"sys_sync",MAX_NAME);
                     break;
    case __NR_kill:  strncpy(ret,"sys_kill",MAX_NAME);
                     break;
    case __NR_rename:  strncpy(ret,"sys_rename",MAX_NAME);
                     break;
    case __NR_mkdir:  strncpy(ret,"sys_mkdir",MAX_NAME);
                     break;
    case __NR_rmdir:  strncpy(ret,"sys_rmdir",MAX_NAME);
                     break;
    case __NR_dup:  strncpy(ret,"sys_dup",MAX_NAME);
                     break;
    case __NR_pipe:  strncpy(ret,"sys_pipe",MAX_NAME);
                     break;
    case __NR_times:  strncpy(ret,"sys_times",MAX_NAME);
                     break;
    case __NR_prof:  strncpy(ret,"sys_prof",MAX_NAME);
                     break;
    case __NR_brk:  strncpy(ret,"sys_brk",MAX_NAME);
                     break;
    case __NR_setgid:  strncpy(ret,"sys_setgid",MAX_NAME);
                     break;
    case __NR_getgid:  strncpy(ret,"sys_getgid",MAX_NAME);
                     break;
    case __NR_signal:  strncpy(ret,"sys_signal",MAX_NAME);
                     break;
    case __NR_geteuid:  strncpy(ret,"sys_geteuid",MAX_NAME);
                     break;
    case __NR_getegid:  strncpy(ret,"sys_getegid",MAX_NAME);
                     break;
    case __NR_acct:  strncpy(ret,"sys_acct",MAX_NAME);
                     break;
    case __NR_umount2:  strncpy(ret,"sys_umount2",MAX_NAME);
                     break;
    case __NR_lock:  strncpy(ret,"sys_lock",MAX_NAME);
                     break;
    case __NR_ioctl:  strncpy(ret,"sys_ioctl",MAX_NAME);
                     break;
    case __NR_fcntl:  strncpy(ret,"sys_fcntl",MAX_NAME);
                     break;
    case __NR_mpx:  strncpy(ret,"sys_mpx",MAX_NAME);
                     break;
    case __NR_setpgid:  strncpy(ret,"sys_setpgid",MAX_NAME);
                     break;
    case __NR_ulimit:  strncpy(ret,"sys_ulimit",MAX_NAME);
                     break;
    case __NR_oldolduname:  strncpy(ret,"sys_oldolduname",MAX_NAME);
                     break;
    case __NR_umask:  strncpy(ret,"sys_umask",MAX_NAME);
                     break;


    /** Add More If Necessary **/

    default: strncpy(ret,"Unknown Syscall",MAX_NAME);
  }

  return ret;
}

int check_syscall(syscall_t *orig, syscall_t *cmp) {
  if (orig->addr != cmp->addr) return ENOADDR;

  if (memcmp(orig->sc_rutine,cmp->sc_rutine,sizeof(orig->sc_rutine)))
    return ENOCODE;

  return SUCCESS;
}

int restore_syscall(syscall_t *st, int mode) {
  unsigned long sc_addr, sct, idt_addr_base, int80;
  unsigned int count=0;
  int fd;
  char sc_asm[CALLOFF], *p;

  idt_addr_base=get_idt_addr(NULL);

  if ((fd=open("/dev/kmem", O_RDWR))==-1) return ERROR;

  readkmem(fd,&idt,idt_addr_base+8*0x80,sizeof(idt));
  int80=(idt.off2 << 16) | idt.off1;

  readkmem(fd,sc_asm,int80,CALLOFF);

  p=(char*)memmem(sc_asm,CALLOFF,"\xff\x14\x85",3);
  sct=*(unsigned*)(p+3);

  if (p)
    if (mode==ENOADDR)
      writekmem(fd, &st->addr, sct + st->nr_syscall * 4, 4);

    if (mode==ENOCODE)
      writekmem(fd, st->sc_rutine, st->addr, 7);

  else {
    errno=ENOSYSCALL;
    close(fd);
    return ERROR;
  }

  close(fd);
  return SUCCESS;
}
