/* main.c  -  Main program
 *      Copyright (C) 2002 fkt <fkt@funfatal.org>
 *
 * This file is part of ProSum.
 *
 * ProSum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ProSum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "include/common.h"
#include "include/stat.h"
#include "include/md5_2.h"
#include "include/main_if.h"

#ifdef HAVE_EXT2FS_EXT2_FS_H
  #include "include/ext2.h"
#endif

#ifdef KSYMS_SUPPORT
  #include "include/ksyms.h"
#endif

#ifdef IDT_SUPPORT
  #include "include/idt.h"
#endif

#ifdef SYSCALL_SUPPORT
  #include "include/syscall.h"
#endif

#define TMP_FILE "/tmp/prosum_bbdd"

int main(int argc, char **argv) {
#ifdef KSYMS_SUPPORT
  char *conf_file=NULL;
#endif

#ifdef HAVE_EXT2FS_EXT2_FS_H
  char ext2_str[14];
#endif

#ifdef IS_BSD
  char filename_zero[MAX_NAME_SZ];
#endif

#if defined(IDT_SUPPORT) || defined(SYSCALL_SUPPORT) || defined (KSYMS_SUPPORT)
  table_t sc_idt_cmp;
#endif

  //Encrypt Vars
  unsigned int encrypt=NO_BD_ENCRYPT, decrypt=NO_BD_DECRYPT;
  CIPHER_HANDLE ch;
  int algo=0;
  size_t ivec_i;
  unsigned int seed=0;
  unsigned char *ivec=NULL, *key;

  int opcion, err;
  unsigned int create=0, report=0, verify=0, req_change=0, changed=0, interactive=0,
               automatic=0, replace=0, conn_bastion_host=0;
  table_t tabla;
  file_t cmp_f;
  char *fich_files, *output_file, *bd_fich, buf[MAX_NAME_SZ], tmp_filename[MAX_NAME_SZ],
       mode_tmp[9], mtime_char[100], *locale, *self_pass=NULL, salt[2];
  FILE *fp_f_fich, *fp_out, *fp_bd, *fp_tmp;
  err_buf_t buf_error_st;
  size_t i, j;
  struct group *gr_st, *gr_st2;
  struct passwd *pw_st, *pw_st2;
  struct tm *m_tm;
  struct utimbuf sync;

#ifdef HAVE_NLS
  //Ajusto el locale
  setlocale(LC_TIME,"");
  if ((locale=setlocale(LC_MESSAGES,""))==NULL) FATAL("setlocale()");
  bindtextdomain(PACKAGE, LOCALE_DIR);
  textdomain(PACKAGE);
  printf(_("\nSetting Locale To: %s\n\n"), locale);
#endif

  signal(SIGINT,signal_handler);
  signal(SIGTERM,signal_handler);

  if (geteuid()) {
    printf(_("You Need ROOT Privileges To Run Program\n"));
    exit(ERROR);
  }

  if (argc < 3) prosum_usage(argv[0]);

  if (check_lock_file(LOCK_FILE)==ERROR) {
    printf(_("%s Already Exists, Other ProSum Is Running\n"), LOCK_FILE);
    printf(_("Or Last Execution Haven't Been Clean Up Correctly\n"));
    printf(_("Please Remove File Manually If You Sure That Another ProSum Isn't Running\n"));
    exit(ERROR);
  }

  if (create_lock_file(LOCK_FILE)==ERROR) FATAL("lock_file()");

  //Registramos la funcion clean_up() para que se ejecute cuando se produzca un exit
  //y asi estamos seguros de que no quedaran archivos temporales
  atexit(clean_up);

  #ifdef KSYMS_SUPPORT
    while ((opcion=getopt(argc,argv,"p:s:f:c:r:v:id:e:a"))!=-1) {
  #else
    while ((opcion=getopt(argc,argv,"p:s:c:r:v:id:e:a"))!=-1) {
  #endif

    switch (opcion) {
#ifdef KSYMS_SUPPORT
      case 'f': if (argc < 5) prosum_usage(argv[0]);

                conf_file=optarg;
                break;
#endif

      case 'p':
                self_pass=optarg;
                break;
      case 's':
                seed=(unsigned int)atoi(optarg);
                break;
      case 'd':
                if (argc < 5) prosum_usage(argv[0]);

                decrypt=BD_DECRYPT;
                key=optarg;
                break;
      case 'e':
                if (argc < 6) prosum_usage(argv[0]);

                encrypt=BD_ENCRYPT;
                key=optarg;
                break;
      case 'v':
                verify=1;
                bd_fich=optarg;
                break;
      case 'c':
                if (argc < 4) prosum_usage(argv[0]);

                create=1;
                output_file=argv[3];
                fich_files=optarg;
                break;
      case 'r':
                report=1;
                bd_fich=optarg;
                break;
      case 'i':
                if (argc < 4) prosum_usage(argv[0]);

                interactive=1;
                break;
      case 'a':  
                if (argc < 4) prosum_usage(argv[0]);
                
                automatic=1;   
                break;
    }
  }

  if (create || interactive) {
    if (!self_pass) {
      printf(_("Specify Password Please\n"));
      exit(ERROR);
    }

    salt[0]=SELF_PASSWD[0];
    salt[1]=SELF_PASSWD[1];
    salt[2]='\0';

    if (strncmp(SELF_PASSWD, crypt(self_pass,salt), strlen(SELF_PASSWD))) {
      printf(_("Incorrect Password!\n"));
      exit(ERROR);
    }
  }

  if ((create && report) || (create && verify) || (report && verify)) {
    printf(_("Please Specify Only ONE Option!\n"));
    exit(ERROR);
  }

#ifdef KSYMS_SUPPORT
  if (!conf_file && (create || verify)) {
    printf(_("Please Specify Configuration File\n"));
    exit(ERROR);
  }
#endif

  if (((encrypt==BD_ENCRYPT) || (decrypt==BD_DECRYPT)) && ((seed == 0) || (seed == 1))) {
    printf(_("Please Specify Seed Random With -s\n"));
    exit(ERROR);
  }

  if ((encrypt==BD_ENCRYPT) && (decrypt==BD_DECRYPT)) {
    printf(_("Please Specify Only -d or -e, Not Both!\n"));
    exit(ERROR);
  }

  if ((encrypt==BD_ENCRYPT) && (report || verify)) {
    printf(_("-e Option Is Incompatible With -r And -v\n"));
    exit(ERROR);
  }

  if ((decrypt==BD_DECRYPT) && create) {
    printf(_("-d Option Is Incompatible With -c\n"));
    exit(ERROR);
  }

  if (automatic && (create || report || interactive)) {
    printf(_("-a Option Is Incompatible With -c, -i And -r\n"));
    exit(ERROR);
  }

  //Comprobamos la llave e inicializamos el CIPHER_HANDLE si es necesario
  if ((encrypt==BD_ENCRYPT) || (decrypt==BD_DECRYPT)) {
    algo=string_to_cipher_algo("BLOWFISH");

    if (!sec_init)
      init_secmem();

    ch=cipher_open(algo, CIPHER_MODE_AUTO_CFB, 1);
    cipher_setkey(ch, key, strlen(key));

    ivec=(char *)malloc(sizeof(char)*cipher_get_blocksize(algo));
    srand(seed);
    //Relleno el ivec
    for (ivec_i=0 ; ivec_i < cipher_get_blocksize(algo) ; ivec_i++)
      ivec[ivec_i]=1+(char) (255.0*rand()/(RAND_MAX+1.0));

    cipher_setiv(ch, ivec, cipher_get_blocksize(algo));

    printf(_("Algo: %s Initiliatized\nKey: %s\nSeed Random: %u\n"),
                                   cipher_algo_to_string(algo), key, seed);

  }

  if (create) {
    tabla.f_validos=0;

    if (file_srv(ALIAS_NAME, SETUP_ALIAS)==ERROR) {
      printf(_("Warning: "));
      if (value_err(errno)==ERROR)
        FATAL("file_srv()");

    }

    if ((fp_f_fich=fopen(fich_files,"rt"))==NULL) FATAL("fopen()");

    if ((fp_out=fopen(output_file,"wb"))==NULL) FATAL("fopen()");

    //Cojo los nombres de los ficheros de fich_files y los proceso
    fgets(buf,sizeof(buf),fp_f_fich);
    while (!feof(fp_f_fich) && tabla.f_validos <= (MAXFILES-1)) {
      supr_retorno_carro(buf);

      //Si es un link simbolico pasamos del fichero
      if (is_symbolic_link(buf)==SUCCESS) {
        printf(_("%s is a symbolic link, rejecting it\n"), buf);
        fgets(buf,sizeof(buf),fp_f_fich);
        continue;
      }

      strncpy(tabla.f_sta[tabla.f_validos].filename,buf,MAX_NAME_SZ);
      tabla.f_sta[tabla.f_validos].filename[MAX_NAME_SZ]='\0';

      if (do_md5sum(&tabla.f_sta[tabla.f_validos])==ERROR) FATAL("do_md5sum()");

      if (do_stat(&tabla.f_sta[tabla.f_validos])==ERROR) FATAL("do_stat()");

#ifdef HAVE_EXT2FS_EXT2_FS_H
      if (do_ext2(&tabla.f_sta[tabla.f_validos])==ERROR) FATAL("do_ext2()");
#endif

      if (file_srv(tabla.f_sta[tabla.f_validos].filename,SEND_FILE)==ERROR) {
        if (value_err(errno)==ERROR)
          perror("file_srv()");

        exit(ERROR);
      }

      tabla.f_validos++;
      fgets(buf,sizeof(buf),fp_f_fich);
    } // FIN while(!feof)

#ifdef KSYMS_SUPPORT
   if (put_ksyms(conf_file, &tabla)==ERROR) FATAL("put_ksyms()");
#endif

#ifdef IDT_SUPPORT
    if (put_idt(&tabla)==ERROR) FATAL("put_idt()");
#endif

#ifdef SYSCALL_SUPPORT
    if (put_syscalls(&tabla)==ERROR)
      if (value_err(errno)==ERROR)
        FATAL("put_syscalls()");
#endif

    //Escribo la BBDD
    write_bd(fp_out,&tabla);

    fclose(fp_f_fich);
    fclose(fp_out);

    //Encripto la BBDD si hiciese falta
    if (encrypt==BD_ENCRYPT)
      if (encrypt_bd(output_file, ch)==ERROR) FATAL("encrypt_bd()");

  } // FIN if (create)

  if (report) {
    //Desencripto la BBDD si hiciese falta
    if (decrypt==BD_DECRYPT)
      if (decrypt_bd(bd_fich, ch)==ERROR) FATAL("decrypt_bd()");

    if ((fp_bd=fopen(bd_fich,"rb"))==NULL) FATAL("fopen()");

    //Recupero los datos de la BBDD
    read_bd(fp_bd,&tabla);

    fclose(fp_bd);

    //Una vez recuperados los datos encripto la BBDD si hiciese falta
    if (decrypt==BD_DECRYPT) {
      cipher_close(ch);
      ch=cipher_open(algo, CIPHER_MODE_CFB, 0);
      cipher_setkey(ch, key, strlen(key));
      cipher_setiv(ch, ivec, cipher_get_blocksize(algo));

      if (encrypt_bd(bd_fich, ch)==ERROR) FATAL("encrypt_bd()");
    }

    printf(_("\n***** Files *****\n"));
    for(i=0; i < tabla.f_validos; i++) {
      printf("------------- %d --------------\n", i+1);
      printf(_("Filename: %s\n"), tabla.f_sta[i].filename);

#ifdef HAVE_EXT2FS_EXT2_FS_H
      value_ext2(tabla.f_sta[i].ext2_flags, ext2_str);
      printf(_("Extended Flags: %s\n"), ext2_str);
#endif

#ifdef IS_BSD
      m_tm=localtime(&tabla.f_sta[i].f_status.st_mtimespec.tv_sec);
#else
      m_tm=localtime(&tabla.f_sta[i].f_status.st_mtime);
#endif

      strftime(mtime_char,sizeof(mtime_char),"%B %d - %A - %H:%M:%S %Z %Y",m_tm);

      pw_st=getpwuid(tabla.f_sta[i].f_status.st_uid);
      gr_st=getgrgid(tabla.f_sta[i].f_status.st_gid);

      printf(_("UID: %d (%s)\tGID: %d (%s)\nSize: %ld Bytes\nMtime: "),
              tabla.f_sta[i].f_status.st_uid, pw_st->pw_name,
              tabla.f_sta[i].f_status.st_gid, gr_st->gr_name,
              tabla.f_sta[i].f_status.st_size
            );
      //Pongo aqui el printf del mtime_char porque no se porque razon en 
      //BSD no me sale nada si lo hago en el printf anterior
      printf(_("%s\n"), mtime_char);

      printf("md5sum: ");
      for(j=0;j < 16;j++)
        printf("%0.2x", tabla.f_sta[i].f_md5[j]);

      printf("\n");

      value_mode(tabla.f_sta[i].f_status.st_mode,mode_tmp);
      printf(_("Mode: "));
      printf("%s\n", mode_tmp);

      printf("-----------------------------\n");
    } // FIN for files

#ifdef KSYMS_SUPPORT
    printf("\n***** KSYMS *****\n");
    for(i=0; i < tabla.ksym_validos; i++) {
      printf(_("KSym Name: %s\t"), tabla.ksym_sta[i].ksym_name);
      printf(_("Address: 0x%x\n"), tabla.ksym_sta[i].addr);
    }
#endif

#ifdef IDT_SUPPORT
    printf("\n***** IDT *****\n");
    for(i=0; i < tabla.idt_validos; i++) {
      printf(_("INT Number: %d\t"), tabla.idt_sta[i].nr_int);
      printf(_("Address: 0x%x\n"), tabla.idt_sta[i].addr);
      printf(_("Segment: %s\t"), value_segment(tabla.idt_sta[i].segment));
      printf(_("Flags: %X\n"), tabla.idt_sta[i].flags);
    }
#endif

#ifdef SYSCALL_SUPPORT
    printf("\n***** Syscalls *****\n");
    for(i=0; i < tabla.sc_validos; i++) {
      printf(_("Syscall: %s\t"), value_syscall(tabla.sc_sta[i].nr_syscall));
      printf(_("Address: 0x%x\n"), tabla.sc_sta[i].addr);
    }
#endif

  } // FIN if (report)

  if (verify) {
    snprintf(tmp_filename,MAX_NAME_SZ,"%s.%d",TMP_FILE,getpid());

    if (copy_file(bd_fich,tmp_filename)==ERROR) FATAL("copy_file()");

    //Desencripto la BBDD si hiciese falta
    if (decrypt==BD_DECRYPT)
      if (decrypt_bd(tmp_filename, ch)==ERROR) FATAL("decrypt_bd()");

    if ((fp_out=fopen(bd_fich,"wb"))==NULL) FATAL("fopen()");
    if ((fp_tmp=fopen(tmp_filename,"rb"))==NULL) FATAL("fopen()");

    //Recupero los datos de la copia de la BBDD y voy reescribiendo en la 
    //BBDD original por si hay algun cambio requerido
    read_bd(fp_tmp,&tabla);

    fclose(fp_tmp);

    //Borro el fichero temporal porque ya no me hace falta
    if (unlink(tmp_filename)==-1) FATAL("unlink()");

    printf(_("\n***** Files *****\n"));
    for(i=0; i < tabla.f_validos; i++) {
      printf("-----------------------------");
      printf(_("\nFile: %s\n"), tabla.f_sta[i].filename);

      //Meto el nombre del fichero en una var auxiliar para luego comparar 
      //los resultados del md5sum etc
      strncpy(cmp_f.filename,tabla.f_sta[i].filename,MAX_NAME_SZ);
      cmp_f.filename[MAX_NAME_SZ]='\0';

      if(do_md5sum(&cmp_f)==ERROR) FATAL("do_md5sum()");
      if(do_stat(&cmp_f)==ERROR) FATAL("do_stat()");

#ifdef HAVE_EXT2FS_EXT2_FS_H
      if (do_ext2(&cmp_f)==ERROR) FATAL("do_ext2()");
#endif

#ifdef IS_BSD
      //En BSD me machaca el filename de tabla.f_sta[0] no se porque
      //por eso lo guardo en una varible auxiliar para luego recuperarlo
      if (!i)
        strncpy(filename_zero,cmp_f.filename,MAX_NAME_SZ);
#endif

#ifdef HAVE_EXT2FS_EXT2_FS_H
      //Compruebo los flags extendidos si hiciera falta
      if (check_ext2(&tabla.f_sta[i],&cmp_f)==ERROR) {
        changed=1;
        value_ext2(tabla.f_sta[i].ext2_flags, ext2_str);
        printf(_("Extended Flags Has Changed!! From: %s"), ext2_str);
        value_ext2(cmp_f.ext2_flags, ext2_str);
        printf(_(" To: %s\n"), ext2_str);
      }
#endif

      //Compruebo el md5sum
      if (check_md5sum(&tabla.f_sta[i],&cmp_f)==ERROR) {
        changed=1;
        conn_bastion_host=1;
        printf(_("MD5 Has Changed!! From: "));
        for(j=0;j < 16;j++)
          printf("%0.2x", tabla.f_sta[i].f_md5[j]);

        printf(_(" To: "));
        for(j=0;j < 16;j++)
          printf("%0.2x", cmp_f.f_md5[j]);

        printf("\n");
      }

      //Pongo el array de errores de stat a SUCCESS entero
      memset(buf_error_st,SUCCESS,sizeof(buf_error_st));

      if (check_stat(&tabla.f_sta[i].f_status,&cmp_f.f_status,buf_error_st)==ERROR) {
        changed=1;
        if (buf_error_st[0]==ERROR) {
          value_mode(tabla.f_sta[i].f_status.st_mode, mode_tmp);
          printf(_("\nMode Has Changed!! From: %s"), mode_tmp);
          value_mode(cmp_f.f_status.st_mode, mode_tmp);
          printf(_(" To: %s\n"), mode_tmp);
        }

        if (buf_error_st[1]==ERROR) {
          pw_st=getpwuid(tabla.f_sta[i].f_status.st_uid);
          printf(_("\nUID Has Changed!! From: %d (%s)"), tabla.f_sta[i].f_status.st_uid,
                                                         pw_st->pw_name);
          pw_st2=getpwuid(cmp_f.f_status.st_uid);
          printf(_(" To: %d (%s)\n"), cmp_f.f_status.st_uid, pw_st2->pw_name);
        }

        if (buf_error_st[2]==ERROR) {
          gr_st=getgrgid(tabla.f_sta[i].f_status.st_gid);
          printf(_("\nGID Has Changed!! From: %d (%s)"), tabla.f_sta[i].f_status.st_gid,
                                                         gr_st->gr_name);
          gr_st2=getgrgid(cmp_f.f_status.st_gid);
          printf(_(" To: %d (%s)\n"), cmp_f.f_status.st_gid, gr_st2->gr_name);
        }

        if (buf_error_st[3]==ERROR) {
          conn_bastion_host=1;
          printf(_("\nSize Has Changed!! From: %ld To: "), tabla.f_sta[i].f_status.st_size);
          //Al BSD no le mola todo en un printf xP
          printf(_("%ld\n"), cmp_f.f_status.st_size);
        }

        if (buf_error_st[4]==ERROR) {
#ifdef IS_BSD
          m_tm=localtime(&tabla.f_sta[i].f_status.st_mtimespec.tv_sec);
#else
          m_tm=localtime(&tabla.f_sta[i].f_status.st_mtime);
#endif

          memset(mtime_char,0x0,sizeof(mtime_char));
          strftime(mtime_char,sizeof(mtime_char),"%B %d - %A - %H:%M:%S %Z %Y",m_tm);
          printf(_("\nLast Modify Has Changed!! From: %s"), mtime_char);
#ifdef IS_BSD
          m_tm=localtime(&cmp_f.f_status.st_mtimespec.tv_sec);
#else
          m_tm=localtime(&cmp_f.f_status.st_mtime);
#endif
          memset(mtime_char,0x0,sizeof(mtime_char));
          strftime(mtime_char,sizeof(mtime_char),"%B %d - %A - %H:%M:%S %Z %Y",m_tm);
          printf(_(" To: %s\n"), mtime_char);
        }

      } //FIN if (check_stat())

      do_questions(&replace,&req_change,changed,automatic,interactive);

      if (!changed) printf(_("Not Changed\n"));

      if (req_change) {
        memcpy(&tabla.f_sta[i],&cmp_f,sizeof(file_t));

        //Si los cambios requieren que cambiemos tambien el fichero en el host bastion...
        if (conn_bastion_host) {
          if (file_srv(tabla.f_sta[i].filename,SEND_FILE)==ERROR) {
            if (value_err(errno)==ERROR)
              perror("file_srv()");

            exit(ERROR);
          }
        } //FIN if (conn_bastion_host)

        printf(_("OK, Changes Saved\n"));
      }

      if (replace) {
        if (conn_bastion_host) {
          if (file_srv(tabla.f_sta[i].filename,RECV_FILE)==ERROR) {
            if (value_err(errno)==ERROR)
              perror("file_srv()");

            exit(ERROR);
          }
        } //FIN if (conn_bastion_host)

        //Cambio el tiempo, el modo, el GID y el UID del archivo para que
        //concuerde con el de la BBDD
        sync.actime=tabla.f_sta[i].f_status.st_atime;
#ifdef IS_BSD
        sync.modtime=tabla.f_sta[i].f_status.st_mtimespec.tv_sec;
#else
        sync.modtime=tabla.f_sta[i].f_status.st_mtime;
#endif
        utime(tabla.f_sta[i].filename,&sync);

        chown(tabla.f_sta[i].filename, tabla.f_sta[i].f_status.st_uid, tabla.f_sta[i].f_status.st_gid);
        chmod(tabla.f_sta[i].filename, tabla.f_sta[i].f_status.st_mode);

#ifdef HAVE_EXT2FS_EXT2_FS_H
        restore_ext2(tabla.f_sta[i].filename, tabla.f_sta[i].ext2_flags);
#endif

        printf(_("OK, Changes Replaced\n"));
      } //FIN if (replace)

      changed=0;
      req_change=0;
      replace=0;
      conn_bastion_host=0;
    } //FIN for

#ifdef KSYMS_SUPPORT
    if (put_ksyms(conf_file, &sc_idt_cmp)==ERROR) FATAL("put_ksyms()");

    printf("\n**** KSYMS ****\n");
    for (i=0; i < tabla.ksym_validos ; i++) {
      printf(_("Checking %s KSym...\n"), tabla.ksym_sta[i].ksym_name);
      if (check_ksym(&tabla.ksym_sta[i],&sc_idt_cmp.ksym_sta[i])==ERROR) {
        printf(_("The Rutine Of %s Has Changed\n"), &tabla.ksym_sta[i].ksym_name);
        changed=1;
      }

      do_questions(&replace,&req_change,changed,automatic,interactive);

      if (!changed) printf(_("Not Changed\n"));

      if (req_change) {
        memcpy(&tabla.ksym_sta[i],&sc_idt_cmp.ksym_sta[i],sizeof(ksym_t));
        printf(_("OK, Changes Saved\n"));
      }

      if (replace) {
        if (restore_ksym(&tabla.ksym_sta[i])==SUCCESS)
          printf(_("OK, Changes Replaced\n"));
        else
          printf(_("ERROR Replacing Changes\n"));
      }

      req_change=0;
      changed=0;
      replace=0;
    } //FIN for
#endif

#ifdef IDT_SUPPORT
    if (put_idt(&sc_idt_cmp)==ERROR) FATAL("put_idt()");

    printf("\n***** IDT *****\n");
    for(i=0; i < tabla.idt_validos; i++) {
      printf(_("Checking INT %d...\n"), tabla.idt_sta[i].nr_int);
      if (check_idt(&tabla.idt_sta[i],&sc_idt_cmp.idt_sta[i])==ERROR) {
        printf(_("Address of INT %d Has Changed From: 0x%x To: 0x%x\n"),
                          tabla.idt_sta[i].nr_int, tabla.idt_sta[i].addr,
                          sc_idt_cmp.idt_sta[i].addr);
        changed=1;
      }

      do_questions(&replace,&req_change,changed,automatic,interactive);

      if (!changed) printf(_("Not Changed\n"));

      if (req_change) {
        memcpy(&tabla.idt_sta[i],&sc_idt_cmp.idt_sta[i],sizeof(idt_t));
        printf(_("OK, Changes Saved\n"));
      }

      if (replace) {
        if (restore_idt(&tabla.idt_sta[i])==SUCCESS)
          printf(_("OK, Changes Replaced\n"));
        else
          printf(_("ERROR Replacing Changes\n"));
      }

      req_change=0;
      changed=0;
      replace=0;
    } //FIN for
#endif

#ifdef SYSCALL_SUPPORT
    if (put_syscalls(&sc_idt_cmp)==ERROR)
      if (value_err(errno)==ERROR)
        FATAL("put_syscalls()");

    printf("\n***** Syscalls *****\n");
    for(i=0; i < tabla.sc_validos; i++) {
      printf(_("Checking %s (%d)...\n"),
                value_syscall(tabla.sc_sta[i].nr_syscall),
                tabla.sc_sta[i].nr_syscall);
      if ((err=check_syscall(&tabla.sc_sta[i],&sc_idt_cmp.sc_sta[i]))==ENOADDR) {
        printf(_("Address of %s (%d) Has Changed From: 0x%x To: 0x%x\n"),
                          value_syscall(tabla.sc_sta[i].nr_syscall),
                          tabla.sc_sta[i].nr_syscall, tabla.sc_sta[i].addr,
                          sc_idt_cmp.sc_sta[i].addr);
        changed=1;
      }

      if (err==ENOCODE) {
        printf(_("The Rutine Of %s (%d) Has Changed\n"),
                                 value_syscall(tabla.sc_sta[i].nr_syscall),
                                 tabla.sc_sta[i].nr_syscall);
        changed=1;
      }

      do_questions(&replace,&req_change,changed,automatic,interactive);

      if (!changed) printf(_("Not Changed\n"));

      if (req_change) {
        memcpy(&tabla.sc_sta[i],&sc_idt_cmp.sc_sta[i],sizeof(syscall_t));
        printf(_("OK, Change Saved\n"));
      }

      if (replace) {
        if (restore_syscall(&tabla.sc_sta[i], err)==SUCCESS)
          printf(_("OK, Changes Replaced\n"));
        else
          printf(_("ERROR Replacing Changes\n"));
      }

      req_change=0;
      changed=0;
      replace=0;
    } //FIN for
#endif

#ifdef IS_BSD
    strncpy(tabla.f_sta[0].filename,filename_zero,MAX_NAME_SZ);
#endif

    write_bd(fp_out,&tabla);
    fclose(fp_out);

    //Encripto la BBDD si hiciese falta
    if (decrypt==BD_DECRYPT) {
      cipher_close(ch);
      ch=cipher_open(algo, CIPHER_MODE_CFB, 0);
      cipher_setkey(ch, key, strlen(key));
      cipher_setiv(ch, ivec, cipher_get_blocksize(algo));

      if (encrypt_bd(bd_fich, ch)==ERROR) FATAL("encrypt_bd()");
    }

  } //FIN if (verify)

  if (algo) {
    free(ivec);
    cipher_close(ch);
  }

  unlink(LOCK_FILE);

  return 0;
}
